#!/usr/bin/env python

import json
import math
import os
import sys


def percentile(items, percent):
    if not items:
        return None
    k = (len(items) - 1) * percent
    f = math.floor(k)
    c = math.ceil(k)
    if f == c:
        return items[int(k)]
    d0 = items[int(f)] * (c - k)
    d1 = items[int(c)] * (k - f)
    return d0 + d1


def calculate_percentiles(items):
    durations = [x['durationInMillis'] for x in items]
    durations = sorted(durations)
    average = sum(durations) / float(len(durations))
    ninty = percentile(durations, 0.9)
    ninty_five = percentile(durations, 0.95)
    ninty_nine = percentile(durations, 0.99)
    return [average, ninty, ninty_five, ninty_nine]


def process_results(results_directory):
    results = {}
    for file in os.listdir(results_directory):
        with open(os.path.join(results_directory, file)) as f:
            data = f.read()
            results[file.replace('.out', '')] = json.loads(data)

    total_millis = 0
    all_requests = []
    total_req_per_second = 0
    failures = 0
    config = None
    print('Server IP\tRequests per second')
    for server in results.keys():
        if config is None:
            config = results[server]['config']
        if 'testError' in results[server]:
            print('{}\t{}'.format(server, results[server]['testError']))
            failures += 1
            continue
        total_millis += results[server]['totalRequestTimeInMillis']
        server_requests = results[server]['requestCount']
        overall_millis = results[server]['overallDurationInMillis']
        req_per_second = 0 if overall_millis == 0 else server_requests / (overall_millis / 1000.0)
        total_req_per_second += req_per_second
        all_requests += results[server]['responses']
        failures += results[server]['failureCount']
        print('{}\t{}'.format(server, req_per_second))

    request_count = len(all_requests)
    percentile_list = calculate_percentiles(all_requests)
    average_millis = -1 if request_count == 0 else percentile_list[0]
    ninety_millis = -1 if request_count == 0 else percentile_list[1]
    ninety_five_millis = -1 if request_count == 0 else percentile_list[2]
    ninety_nine_millis = -1 if request_count == 0 else percentile_list[3]
    num_clients = len(results.keys())
    num_iterations = config['iterationsPerThread']
    num_threads = config['numThreads']
    protocol_string = config['protocolString']
    delay = config['delay']

    results_template = '{}\t{}\t{}\t{}\t{}\t{:8}\t{}\t{:<8.2f}\t{:<8.2f}\t{:.0f}\t{:.0f}\t{:.0f}'
    print('Failures: ' + str(failures))
    print('\nClients\tThreads\tIters\tDelay\tProto\tTotal ms\tCount\tAverage ms\tReqs per sec\t90% ms\t95% ms\t99% ms')
    print(results_template.format(num_clients, num_threads, num_iterations, delay, protocol_string,
                                  str(total_millis), request_count, average_millis, total_req_per_second,
                                  ninety_millis, ninety_five_millis, ninety_nine_millis))


if __name__ == '__main__':
    process_results(sys.argv[1])
